﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Algorithms.DataAnalysis;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

// ReSharper disable once CheckNamespace
namespace HeuristicLab.Algorithms.EGO {

  [StorableClass]
  [Item("NeighbourDistance", "Exploration by maximizing the distance to the nearest neighbour")]
  public class NeighbourDistance : InfillCriterionBase {

    private IVantagePointTree<IEnumerable<double>> Points;
    #region Constructors, Serialization and Cloning 
    [StorableConstructor]
    protected NeighbourDistance(bool deserializing) : base(deserializing) { }

    protected NeighbourDistance(NeighbourDistance original, Cloner cloner) : base(original, cloner) {
    }
    public NeighbourDistance() { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new NeighbourDistance(this, cloner);
    }
    #endregion

    public override double Evaluate(RealVector vector) {
      if (Points == null) Points = CreateTree();
      IList<IEnumerable<double>> neighbours;
      IList<double> distances;
      Points.Search(vector, 1, out neighbours, out distances);
      return distances[0];
      //return RegressionSolution.ProblemData.AllIndices.Min(i => DistanceToSample(vector, i));
    }

    public override void Initialize() {
      Points = CreateTree();
    }

    private double DistanceToSample(RealVector vector, int i) {
      throw new NotImplementedException();
    }

    private VantagePointTree<IEnumerable<double>> CreateTree() {
      var data = RegressionSolution.ProblemData.Dataset;
      var rows = RegressionSolution.ProblemData.AllIndices;
      var cols = RegressionSolution.ProblemData.AllowedInputVariables;
      return new VantagePointTree<IEnumerable<double>>(new EuclideanDistance(), rows.Select(r => cols.Select(c => data.GetDoubleValue(c, r)).ToArray()));
    }
  }
}
